/*
   Erstellt:  2021-06-13
   Autor:  Roland Kaiser, OE2ROL
   geschrieben für Arduino Nano ATmega 328P(Old Bootloader)
   Der Autor übernimmt keine Haftung etwaiger Folgeschäden an Transceivern oder dgl.
   
   Diese Software ist für den eigenen privaten Gebrauch frei verfügbar, jedoch nicht für kommerzielle Zwecke ohne Genehmigung des Autors.
   Veränderungen und Verbesserungen der Software bitte wieder an den Autor Roland Kaiser OE2ROL (roland.kaiser@sbg.at) senden.
   
   Die Software wurde am Yaesu FT-991A getestet, sollte auch am FT-991 funktionieren.

   Funktionsweise:
   
   o  Der Transceiver wird beim Start automatisch in den entsptechenden "SAT Mode" gebracht (beide VFO´s in die vorgegebene QRG, Betriebsart USB, Splitmode, RX auf 70cm, TX auf 2m, HF-Power 5W)
   o  Der 70cm VFO wird gelesen, davon wird ein vorgegebener Versatz subtrahiert und verzögert automatisch in den 2m VFO geschrieben.
   o  Dies passiert vollautomatisch nur durch verändern des 70cm VFO´s.
   o  Die für QO-100 umgerechnete RX und TX Frequenz wird auf einem LCD Display angezeigt. 
   o  RIT mit Tasten up oder down um 100 Hz verstellen
   o  Menüsteuerung mit 4 Ebenen:
       o Setzen eines RX-Offsets 100 Hz Schrittweite zum Anpassen von Ungenauigkeiten des Upconverters
       o Setzen eines TX-Offsets  10 Hz Schrittweite zum Anpassen von Ungenauigkeiten des Downconverters
       o Setzen eines Split-Offsets 1000Hz Schrittweite für Stationen die im Split-Betrieb arbeiten
       o Speichern der RX/TX Offsets im internen EEPROM.
   o  Nach dem Speichern der Offsets wird bei einem Neustart der SAT BOX auf die Frequenz der mittleren QO-100 Bake gestellt.
   o  Die RX und TX Offsets bleiben auch ohne Stromversorgung gespeichert, der Split Offset wird wieder auf 0 gesetzt.
   o  Nach erneuten Start werden die Offsets automatisch aus dem EEPROM ausgelesen und für die Korrektur des Up- und Down-Konverters wieder verwendet.

   Anschlussbelegung der Ports:
   
   RS232 Radio CAT port GND -> Arduino GND
   RS232 Radio CAT port TX  -> Arduino pin 11
   RS232 Radio CAT port RX  -> Arduino pin 12

   Tastenport up      -> Arduino Pin A3
   Tastenport down    -> Arduino Pin A4
   Tastenport Menü    -> Arduino Pin A5
   Die Tasten werden gegen Masse angeschlossen.

   Display:     Arduino:
   4(RS)    ->  7
   6(E)     ->  10
   11(D4)   ->  5
   12(D5)   ->  4
   13(D6)   ->  6
   14(D7)   ->  2
*/

#include <LiquidCrystal.h>
#include <EEPROM.h>
#include <SoftwareSerial.h>

SoftwareSerial RS232(11, 12);         // RS232 TX Pin 11, RX Pin 12
LiquidCrystal lcd(7, 10, 5, 4, 6, 2); // LCD Anschlussbelegung
 
uint32_t cat_rec_vfo_a = 0;       // empfangene Frequenz von VFO A
uint32_t cat_rec_vfo_b = 0;       // empfangene Frequenz von VFO A
byte lcdNumCols = 20;             // LCD Anzahl der Stellen
uint32_t VFO2;                    // zu sendende 2m VFO Frequenz    mit  tx_vfo_b(); senden
uint32_t VFO70;                   // zu sendende 70cm VFO Frequenz  mit  tx_vfo_a(); senden
uint32_t TXfreq;                  // umgerechnete Sendefrequenz zur Anzeige am LCD Display
uint32_t RXfreq;                  // umgerechnete Empfangsfrequenz zur Anzeige am LCD Display
uint32_t TXosz;                   // TX Oszillatorfrequenz
uint32_t RXosz;                   // RX Oszillatorfrequenz
uint32_t shift;                   // 2m 70cm Versatz
uint32_t freqmem;                 // FrequenzSpeicher
long RIT = 0;                     // RIT
long TXoffset;                    // TX Offset
long RXoffset;                    // RX Offset
long Splitoffset;                 // Split Offset
const int menuPin = A5;           // Pin für Menue Taste
const int downpin = A4;           // Pin für Up Taste
const int uppin = A3;             // Pin für Down Taste
int menuPushCounter = 0;          // Menütastenzähler
int menuState = 0;                // Menüstatus
int lastmenuState = 0;            // letzter Menü Status
boolean menubool = false;         // Menü initial deaktivieren
int upState;                      // up Status 
int lastupState;                  // letzter up Status
int up;                           // up Taste
int downState;                    // down Status
int lastdownState;                // letzter down Status
int down;                         // Taste down
int menucounter = 0;              // Menü Zähler
unsigned long previousMillis = 0; // Timer auf 0 setzen
const long interval = 1;          // Verzögerungszeit [ms]zum VFO nachziehen

                
void setup() {

/*
// ************   Wird nur bei der  1. Initialisierung einen neuen Arduino verwendet dann deaktivieren um die Offsets aus dem eeprom zu verwenden **************
TXoffset=0;                                     // TX Offset setzen
RXoffset=0;                                     // RX Offset setzen
EEPROM.put(0, TXoffset);                        // TX Offset Wert 0 ins eeprom schreiben (Adresse 0) 
delay(200);                                     // Verzögerung 200 ms
EEPROM.put(10, RXoffset);                       // Wert ins eeprom schreiben (Adresse 10)
*/

TXosz=2256000000;                               // TX Oszillatorfrequenz [Hz]    (z.B. 2256MHz für 2m)
RXosz=1005770000;                               // RX Oszillatorfrequenz [Hz*10] (z.B. 10,0577GHz 70cm)

pinMode(menuPin, INPUT);                        // Init für die Tasten Pins
digitalWrite(menuPin, HIGH);                    // Init für die Tasten Pins
pinMode(downpin, INPUT);                        // Init für die Tasten Pins
digitalWrite(downpin, HIGH);                    // Init für die Tasten Pins
pinMode(uppin, INPUT);                          // Init für die Tasten Pins
digitalWrite(uppin, HIGH);                      // Init für die Tasten Pins

lcd.begin(2, lcdNumCols);                       // LCD init
lcd.clear();                                    // LCD löschen
delay(100);                                     // Verzögerung

// Initmeldung
lcd.setCursor(0, 0);                            // 1. Zeichen, 1. Zeile
lcd.print(" **** SAT  BOX ****");               // schreiben
lcd.setCursor(0, 1);                            // 1. Zeichen, 2. Zeile
lcd.print("  **** OE2ROL ****");                // schreiben
delay(2000);                                    // Wartezeit
lcd.clear();                                    // LCD löschen

// CAT Settings beim Start setzen

Serial.begin(9600);                             // Serial.begin(9600) für Testzwecke 
while (!Serial);                                // wenn Serielle Schnittstelle verfügbar
RS232.begin(9600);                              // Init Software Serial
delay(1000);                                    // Verzögerung 1000 ms

int dly = 100;                                  // Verzögerung für x millisekunden zwischen den Kommandos
RXoffset = EEPROM.get(10, RXoffset);            // RX Offset [Hz] aus dem eeprom lesen
RS232.print("QS;");                             // Split
delay(dly);                                     // Verzögerung dly [ms]
RS232.print("PC005;");                          // Power 5W
delay(dly);                                     // Verzögerung dly [ms]
RS232.print("BA;");                             // VFO B zu A
delay(dly);                                     // Verzögerung dly [ms]
VFO70=432050000+(RXoffset);                     // VFO A setzen
tx_vfo_a();                                     // VFO A schreiben
delay(dly);                                     // Verzögerung dly [ms]
RS232.print("MD02;");                           // Mode USB
delay(dly);                                     // Verzögerung dly [ms]
RS232.print("PC005;");                          // Power 5W
delay(dly);                                     // Verzögerung dly [ms]
RS232.print("AB;");                             // VFO A zu B
delay(dly);                                     // Verzögerung dly [ms]
VFO2=144500000;                                 // VFO B setzen
tx_vfo_b();                                     // VFO B schreiben
delay(dly);                                     // Verzögerung dly [ms]
RS232.print("MD02;");                           // Mode USB
delay(dly);                                     // Verzögerung dly [ms]
RS232.print("PC005;");                          // Power 5W
delay(dly);                                     // Verzögerung dly [ms]


}
// LCD Beschriftung setzen

void LCDBeschriftung(void) {
lcd.setCursor(0, 0);      // 1. Zeichen, 1. Zeile
lcd.print("RX");          // schreiben
lcd.setCursor(17, 0);     // 18. Zeichen, 1. Zeile
lcd.print("MHz");         // schreiben
lcd.setCursor(0, 1);      // 1. Zeichen, 2. Zeile
lcd.print("TX");          // schreiben
lcd.setCursor(17, 1);     // 18. Zeichen, 2. Zeile
lcd.print("MHz");         // schreiben
//lcd.setCursor(18, 2);   // 1. Zeichen, 3. Zeile
//lcd.print("FWD");       // schreiben
//lcd.setCursor(18, 3);   // 1. Zeichen, 4. Zeile
//lcd.print("REF");       // schreiben
}

void calculateFrequency(void)  {
TXoffset = EEPROM.get(0, TXoffset);               // TX Offset zum Kalibrieren der Sendefrequenz [Hz] aus dem eeprom lesen (Adresse 0)
RXoffset = EEPROM.get(10, RXoffset);              // RX Offset zum Kalibrieren der Empfangsfrequenz [Hz] aus dem eeprom lesen (Adresse 10)
shift=287800000-(TXoffset)+(RXoffset);            // 2m 70cm Versatz [Hz*10] = TX 2400,250MHz RX 10489,750MHz = RX-TX shift 8,089.500GHz = Diff Uplink und Downlink QO100
TXfreq=(VFO2)+(TXosz)-(TXoffset);                 // Berechnung der Sendefrequenz (Sendefreq.2m + Oszillatorfreq.TX)
RXfreq=(VFO70/10)+(RXosz)-(RXoffset/10);          // Berechnung der Empfangsfrequenz (Empanfsfreq.70cm + Oszillatorfreq.RX)
}

//display TX Frequenz (Uplink)

void displayTXfreq(void) {
lcd.setCursor(4, 1);
lcd.print(TXfreq/1000000000%10);   // Tx 1G Stelle ausgeben
lcd.setCursor(5, 1);
lcd.print(TXfreq/100000000%10);    // Tx 100M  Stelle ausgeben
lcd.setCursor(6, 1);
lcd.print(TXfreq/10000000%10);     // Tx 10M Stelle ausgeben
lcd.setCursor(7, 1);
lcd.print(TXfreq/1000000%10);      // Tx 1M Stelle ausgeben
lcd.setCursor(8, 1);
lcd.print(",");                    // , ausgeben
lcd.setCursor(9, 1);
lcd.print(TXfreq/100000%10);       // Tx 100k Stelle ausgeben
lcd.setCursor(10, 1);
lcd.print(TXfreq/10000%10);        // Tx 10k Stelle ausgeben
lcd.setCursor(11, 1);
lcd.print(TXfreq/1000%10);         // Tx 1k Stelle ausgeben
lcd.setCursor(12, 1);
lcd.print(TXfreq/100%10);          // Tx 100Hz Stelle ausgeben
lcd.setCursor(13, 1);
lcd.print(TXfreq/10%10);           // Tx 10Hz Stelle ausgeben
lcd.setCursor(14, 1);
lcd.print(TXfreq%10);              // Tx 1Hz Stelle ausgeben
}

//display RX Frequenz (Downlink)

void displayRXfreq(void) {
lcd.setCursor(1, 2);
lcd.print(RXfreq/1000000000%10);   // Rx 10G Stelle ausgeben
lcd.setCursor(2, 2);
lcd.print(RXfreq/100000000%10);    // Rx 1G Stelle ausgeben
lcd.setCursor(3, 2);
lcd.print(RXfreq/10000000%10);     // Rx 100M Stelle ausgeben
lcd.setCursor(4, 2);
lcd.print(RXfreq/1000000%10);      // Rx 10M Stelle ausgeben
lcd.setCursor(5, 2);
lcd.print(RXfreq/100000%10);       // Rx 1M Stelle ausgeben
lcd.setCursor(6, 2);
lcd.print(",");                    // , ausgeben
lcd.setCursor(7, 2);
lcd.print(RXfreq/10000%10);        // Rx 100k Stelle ausgeben
lcd.setCursor(8, 2);
lcd.print(RXfreq/1000%10);         // Rx 10k Stelle ausgeben
lcd.setCursor(9, 2);
lcd.print(RXfreq/100%10);          // Rx 1k Stelle ausgeben
lcd.setCursor(10, 2);
lcd.print(RXfreq/10%10);           // Rx 100Hz Stelle ausgeben
lcd.setCursor(11, 2);
lcd.print(RXfreq%10);              // Rx 10Hz Stelle ausgeben
lcd.setCursor(12, 2);
lcd.print(RXfreq%1);               // Rx 1Hz Stelle ausgeben
}


void rx_vfo_a() {                                  // VFO A Frequenz vom Transceiver auslesen und in Variable (cat_rec_vfo_a) schreiben

long vfo_a_in_buffer[13];                          // Array zum Einlesen der Frequenz VFO A
int j = 0;                                         // Array Indexzähler auf 0 setzen

  RS232.print("FA;");                              // CAT Kommando zu lesen der Frequenz von VFO A
  delay(20);                                       // Verzögerung 20 ms
  j = 0;                                           // Array Indexzähler auf 0 setzen
  while (!RS232.available());                      // auf Serial Port warten
    while (RS232.available() > 0) {                // wenn Serial Port verfügbar
      vfo_a_in_buffer[j] = RS232.read();           // Zeichen vom Serial Port schrittweise in das Array schreiben (gelesene Daten z.B. FB432050000;)
      j ++;                                        // Arrayindexzähler hochzählen
      }
      vfo_a_in_buffer[j] = 0;                      // Arrayindexzähler auf 0 setzen

cat_rec_vfo_a=((vfo_a_in_buffer[2]-48)*100000000)+ // 1 Stelle 100 MHZ Digit in ASCII Dezimal - 48 = entsprechende Zahl....
              ((vfo_a_in_buffer[3]-48)*10000000)+  // 2 Stelle 10 MHZ
              ((vfo_a_in_buffer[4]-48)*1000000)+   // 3 Stelle 1 MHZ
              ((vfo_a_in_buffer[5]-48)*100000)+    // 4 Stelle 100 KHZ
              ((vfo_a_in_buffer[6]-48)*10000)+     // 5 Stelle 10 KHZ
              ((vfo_a_in_buffer[7]-48)*1000)+      // 6 Stelle 1 KHZ
              ((vfo_a_in_buffer[8]-48)*100)+       // 7 Stelle 100 HZ
              ((vfo_a_in_buffer[9]-48)*10)+        // 8 Stelle 10 HZ
              (vfo_a_in_buffer[10]-48);            // 9 Stelle 1 HZ

}

void rx_vfo_b() {                                  // VFO B Frequenz vom Transceiver auslesen und in Variable (cat_rec_vfo_b) schreiben
  
long vfo_b_in_buffer[13];                          // Array zum Einlesen der Frequenz VFO B
int k = 0;                                         // Array Indexzähler auf 0 setzen

  RS232.print("FB;");                              // CAT Kommando zu lesen der Frequenz von VFO B
  delay(20);                                       // Verzögerung 20 ms
  k = 0;                                           // Array Indexzähler auf 0 setzen
  while (!RS232.available());                      // auf Serial Port warten


    while (RS232.available() > 0) {                // wenn Serial Port verfügbar
      vfo_b_in_buffer[k] = RS232.read();           // Zeichen vom Serial Port schrittweise in das Array schreiben (gelesene Daten z.B. FB144500000;)
      k ++;                                        // Arrayindexzähler hochzählen
      }
      vfo_b_in_buffer[k] = 0;                      // Arrayindexzähler auf 0 setzen
     
cat_rec_vfo_b=((vfo_b_in_buffer[2]-48)*100000000)+ // 1 Stelle 100 MHZ Digit in ASCII Dezimal - 48 = entsprechende Zahl....
              ((vfo_b_in_buffer[3]-48)*10000000)+  // 2 Stelle 10 MHZ
              ((vfo_b_in_buffer[4]-48)*1000000)+   // 3 Stelle 1 MHZ
              ((vfo_b_in_buffer[5]-48)*100000)+    // 4 Stelle 100 KHZ
              ((vfo_b_in_buffer[6]-48)*10000)+     // 5 Stelle 10 KHZ
              ((vfo_b_in_buffer[7]-48)*1000)+      // 6 Stelle 1 KHZ
              ((vfo_b_in_buffer[8]-48)*100)+       // 7 Stelle 100 HZ
              ((vfo_b_in_buffer[9]-48)*10)+        // 8 Stelle 10 HZ
              (vfo_b_in_buffer[10]-48);            // 9 Stelle 1 HZ

}

void tx_vfo_a() {                                  // die Variable (VFO70) in den Transceiver in den VFO A schreiben
  
long vfo_a_buffer[10];                             // Array zum Schreiben der Frequenz in VFO A

for (int i = 0; i < 10; i++) {                     // Zählschleife 0-9
     vfo_a_buffer[i] = VFO70-1;                    // VFO Frequenz in das Array schreiben
     vfo_a_buffer[i]++;                            // Arrayindexzähler hochzählen
     delay(10);                                    // Verzögerung 10 ms
     RS232.print("FA");                            // CAT Kommandoteil VFO A schreiben
     RS232.print(vfo_a_buffer[i]);                 // CAT Kommandoteil Frequenz schreiben
     RS232.print(";");                             // CAT Kommandoteil Stopbit
     vfo_a_buffer[i] = 0;                          // Arrayindexzähler auf 0 setzen
     }
}

void tx_vfo_b() {                                  // die Variable (VFO2) in den Transceiver in den VFO B schreiben
  
long vfo_b_buffer[10];                             // Array zum Schreiben der Frequenz in VFO A

for (int i = 0; i < 10; i++) {                     // Zählschleife 0-9
     vfo_b_buffer[i] = VFO2-1;                     // VFO Frequenz in das Array schreiben
     vfo_b_buffer[i]++;                            // Arrayindexzähler hochzählen
     delay(10);                                    // Verzögerung 10 ms
     RS232.print("FB");                            // CAT Kommandoteil VFO B schreiben
     RS232.print(vfo_b_buffer[i]);                 // CAT Kommandoteil Frequenz schreiben
     RS232.print(";");                             // CAT Kommandoteil Stopbit
     vfo_b_buffer[i] = 0;                          // Arrayindexzähler auf 0 setzen
     }
}

// Nachziehen des 2.VFO

void VFOnachziehen() {
int dly = 10;                                             // Verzögerung für x millisekunden zwischen den Kommandos
rx_vfo_a();                                               // Frequenz VFO A aus dem Transceiver lesen
delay(dly);                                               // Verzögerung
       VFO2=(cat_rec_vfo_a)-(shift)-(RIT)+(Splitoffset);  // VFO Freq. für 2m berechnen und um RIT korrigieren
         unsigned long currentMillis = millis();          // Wert für aktuellen Timer setzen 
          if (currentMillis - previousMillis >= interval) // wenn Differenz des aktuellen Timers zu Startwert des Timers > Verzögerungszeit ist, dann:
            {
               previousMillis = currentMillis;            // Timerstartwert auf Wert des aktuellen Timers setzen
               if (freqmem!=VFO2)                         // Nur wenn der Zwischenspeicher nicht dem VFO der 2m Frequenz entspricht, dann:
                  {
                   delay(dly);                            // Verzögerung
                   tx_vfo_b();                            // VFO mit der neuen 2m Frequenz beschreiben
                   freqmem=VFO2;                          // 2m Frequenz in den Zwischenspeicher schreiben
                   delay(dly);                            // Verzögerung
                  } 
            }
delay(dly);                                               // Verzögerung
VFO70 = cat_rec_vfo_a;                                    // VFO Freq. für 70cm vom Transceiver lesen
tx_vfo_a;                                                 // VFO Freq. für 70cm in den Transceiver schreiben
}

void RXkorr()
{
up = digitalRead(uppin);                        // up Taste abfragen
            if (up == LOW)                      // wenn up Taste betätigt
            {
              delay(200);                       // Verzögerung
              RIT=RIT+100;                      // RIT um 100 Hz erhöhen
              VFO70=((VFO70)+100);              // VFO70 um 100Hz erhöhen
              tx_vfo_a();                       // die Variable (VFO70) in den Transceiver in den VFO A schreiben
              lcd.clear();                      // LCD löschen
              lcd.setCursor(0, 0);              // 1. Zeichen, 1. Zeile
              lcd.print("RIT [Hz]");            // schreiben
              lcd.setCursor(0, 1);              // 1. Zeichen, 2. Zeile
              lcd.print(RIT);                   // schreiben  
              delay(500);                       // Verzögerung
              lcd.clear();                      // LCD löschen
            }
down = digitalRead(downpin);                    // down Taste abfragen
            if (down == LOW)                    // wenn down Taste betätigt
            {
              delay(200);                       // Verzögerung
              RIT=RIT-100;                      // RIT um 100 Hz vermindern
              VFO70=((VFO70)-100);              // VFO70 um 100Hz erhöhen
              tx_vfo_a();                       // die Variable (VFO70) in den Transceiver in den VFO A schreiben
              lcd.clear();                      // LCD löschen
              lcd.setCursor(0, 0);              // 1. Zeichen, 1. Zeile
              lcd.print("RIT [Hz]");            // schreiben
              lcd.setCursor(0, 1);              // 1. Zeichen, 2. Zeile
              lcd.print(RIT);                   // schreiben
              delay(500);                       // Verzögerung
              lcd.clear();                      // LCD löschen
            }
}
  
void loop() {         
int dly = 5;                            // Verzögerung für x millisekunden zwischen den Kommandos
LCDBeschriftung();                      // LCD Beschriftung ausführen
calculateFrequency();                   // Frequenzberechnung durchführen
displayTXfreq();                        // TX Frequenz ins Display schreiben
displayRXfreq();                        // RX Frequenz ins Display schreiben
RXkorr();                               // RIT ausführen
VFOnachziehen();                        // VFO nachziehen

//Serial.println(RXfreq);               // nur für Testzwecke
//Serial.println(TXfreq);               // nur für Testzwecke
//Serial.println(VFO70);                // nur für Testzwecke
//Serial.println(VFO2);                 // nur für Testzwecke
//Serial.println(TXoffset);             // nur für Testzwecke
//Serial.println(RXoffset);             // nur für Testzwecke
//Serial.println(cat_rec_vfo_a);        // nur für Testzwecke
//Serial.println(cat_rec_vfo_b);        // nur für Testzwecke
//Serial.println(RIT);                  // nur für Testzwecke

delay(dly);                             // Verzögerung

menuState = digitalRead(menuPin);                       // prüfen ob die Menütaste gedrückt ist
  if (menuState != lastmenuState)                       // wenn sich der Menü Status geändert hat
  {
    if (menuState == LOW)                               // wenn Menü Status LOW
    {
      delay(200);                                       // Verzögerung
      menuPushCounter = 4;                              // maximal 4 Menüs verfügbar Menütastenzähler auf 4 setzen
      menubool = true;                                  // menü aktivieren
    }
    if (menuState == HIGH)                              // wenn Menü Status HIGH
    {
      menubool = false;                                 // menü deaktivieren
    }
  }

  if (menubool == true && menuPushCounter == 4)         // wenn Menü aktiviert und Menütastenzähler 4
  {
    menucounter = 0;                                    // Menüzähler auf 0 setzen 
    menu();                                             // Menü starten
  }
lastmenuState = menuState;
}

// Menüsteuerung zum Setzen und Speichern der Offsets

int menu()
{
  lcd.clear();                                    // LCD löschen
  delay(100);                                     // Verzögerung
  while (menucounter < 4)                         // while Menu enspricht der max Anzahl der Menüs
  {
    menuState = digitalRead(menuPin);             // Menü Taste abfragen
    up = digitalRead(uppin);                      // up Taste abfragen
    down = digitalRead(downpin);                  // down Taste abfragen

    if (menuState != lastmenuState)               // wenn sich der Menü Status geändert hat
    {
      lcd.clear();                                // LCD löschen
      if (menuState == LOW)                       // wenn Menü Status LOW
      {
        delay(200);                               // Verzögerung
        menucounter++;                            // Menüzähler um 1 erhöhen
      }
    }
lastmenuState = menuState;                        // letzten Menüstatus auf Menü Status setzen
   
                                                  // zeigt die Menüs

    switch (menucounter) {                        // Menüauswahl
      case 0:                                     // Menü 1 TX Offset
        lcd.setCursor(0, 0);                      // 1. Zeichen, 1. Zeile
        lcd.print("TX Offset [Hz]");              // schreiben
        lcd.setCursor(0, 1);                      // 1. Zeichen, 2. Zeile
        lcd.print(TXoffset);                      // schreiben
            if (up == LOW)                        // wenn up Taste betätigt
            {
              delay(200);                         // Verzögerung
              TXoffset=TXoffset+10;               // Wert um 10 erhöhen
              lcd.clear();                        // LCD löschen
            }
            if (down == LOW)                      // wenn down Taste betätigt
            {
              delay(200);                         // Verzögerung
              TXoffset=TXoffset-10;               // Wert um 10 vermindern
              lcd.clear();                        // LCD löschen
            }
        break;                                    // aussteigen, dieses Menü beginnt dann von vorne

      case 1:                                     // Menü 2 RX Offset
        lcd.setCursor(0, 0);                      // 1. Zeichen, 1. Zeile
        lcd.print("RX Offset [Hz]");              // schreiben
        lcd.setCursor(0, 1);                      // 1. Zeichen, 2. Zeile
        lcd.print(RXoffset);                      // schreiben
            if (up == LOW)                        // wenn up Taste betätigt
            {
              delay(200);                         // Verzögerung
              RXoffset=RXoffset+100;              // Wert um 100 erhöhen
              lcd.clear();                        // LCD löschen
            }
            if (down == LOW)                      // wenn down Taste betätigt
            {
              delay(200);                         // Verzögerung
              RXoffset=RXoffset-100;              // Wert um 100 vermindern
              lcd.clear();                        // LCD löschen
            }
        break;                                    // aussteigen, dieses Menü beginnt dann von vorne
        
      case 2:                                     // Menü 2 Split Offset
        lcd.setCursor(0, 0);                      // 1. Zeichen, 1. Zeile
        lcd.print("Split Offset [Hz]");           // schreiben
        lcd.setCursor(0, 1);                      // 1. Zeichen, 2. Zeile
        lcd.print(Splitoffset);                   // schreiben
            if (up == LOW)                        // wenn up Taste betätigt
            {
              delay(200);                         // Verzögerung
              Splitoffset=Splitoffset+1000;       // Wert um 1000 erhöhen
              lcd.clear();                        // LCD löschen
            }
            if (down == LOW)                      // wenn down Taste betätigt
            {
              delay(200);                         // Verzögerung
              Splitoffset=Splitoffset-1000;       // Wert um 1000 vermindern
              lcd.clear();                        // LCD löschen
            }
        break;                                    // aussteigen, dieses Menü beginnt dann von vorne
        
      case 3:                                     // Menü 3, hier werden die Werte gespeichert und der arduino beginnt loop neu auszuführen
        lcd.setCursor(0, 0);                      // 1. Zeichen, 1. Zeile
        lcd.print("store RX/TX Offsets?");        // schreiben
        lcd.setCursor(0, 1);                      // 1. Zeichen, 2. Zeile
        lcd.print("down= No     up= Yes");        // schreiben
            if (up == LOW)                        // wenn up Taste betätigt
            {
              delay(200);                         // Verzögerung
              EEPROM.put(0, TXoffset);            // Wert ins eeprom schreiben (Adresse 0)
              delay(200);                         // Verzögerung
              EEPROM.put(10, RXoffset);           // Wert ins eeprom schreiben (Adresse 10)
              delay(200);                         // Verzögerung
              lcd.setCursor(0, 1);                // 1. Zeichen, 1. Zeile
              lcd.clear();                        // LCD löschen
              lcd.print("Offsets stored");        // schreiben
              delay(2000);                        // Verzögerung
              lcd.clear();                        // LCD löschen
              delay(200);                         // Verzögerung
              menucounter = 4;                    // Menü Zähler auf 4 setzen
              menuPushCounter = 0;                // Menütastenzähler auf 0 setzen
              break;                              // aussteigen, zurück zu loop
            }
            if (down == LOW)                      // wenn down Taste betätigt
            {
              lcd.clear();                        // LCD löschen
              delay(200);                         // Verzögerung
              menucounter = 4;                    // Menü Zähler auf 4 setzen
              menuPushCounter = 0;                // Menütastenzähler auf 0 setzen
              break;                              // aussteigen, zurück zu loop
            }
    }
  }
}
